package com.alentoma.selectabletext;

import static com.alentoma.selectabletext.Utils.pxToDp;

import android.content.Context;
import android.text.Layout;
import android.util.Log;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.ArrayList;
import java.util.List;

public class RNSelectableTextRectHelperModule extends ReactContextBaseJavaModule {
    public static final String REACT_CLASS = "RNSelectableTextRectHelper";

    RNSelectableTextRectHelperModule(ReactApplicationContext context) {
        super(context);
    }

    @NonNull
    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @ReactMethod
    public void getRectForSubstring(int reactTag, int selectionStart, int selectionEnd, Promise promise) {
        try {
            TextView selectableText = getReactApplicationContext().getCurrentActivity().findViewById(reactTag);
            WritableArray rects = Arguments.createArray();

            if (selectableText != null) {
                Layout layout = selectableText.getLayout();
                if (layout != null) {
                    Context context = selectableText.getContext();
                    int startLine = layout.getLineForOffset(selectionStart);
                    int endLine = layout.getLineForOffset(selectionEnd);


                    for (int line = startLine; line <= endLine; line++){

                        int lineStart = (int) (line == startLine ? layout.getPrimaryHorizontal(selectionStart) : layout.getLineLeft(line));
                        int lineEnd = (int) (line == endLine ? layout.getPrimaryHorizontal(selectionEnd) : layout.getLineRight(line));

                        int top = layout.getLineTop(line);
                        int bottom = layout.getLineBottom(line);

                        WritableMap rect = Arguments.createMap();
                        rect.putInt("x", pxToDp(context, lineStart));
                        rect.putInt("y", pxToDp(context, top));
                        rect.putInt("width", pxToDp(context, lineEnd - lineStart));
                        rect.putInt("height", pxToDp(context, bottom - top));
                        rects.pushMap(rect);
                    }
                }
            }

            WritableMap result = Arguments.createMap();
            result.putArray("rects", rects);

            promise.resolve(result);
        } catch (Exception e) {
            promise.reject(e);
        }
    }
}
