package com.alentoma.selectabletext;

import android.content.Context;
import android.view.ActionMode;
import android.view.Menu;

import androidx.appcompat.widget.TintContextWrapper;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.facebook.react.views.text.ReactTextView;

/*
    Custom wrapper component that would allow us to observe selection changes.
*/
public class TextViewSelectionObserver extends ReactTextView {
    public TextViewSelectionObserver(Context context) {
        super(context);
    }

    private ReactContext getReactContext() {
        Context context = this.getContext();
        return context instanceof TintContextWrapper ? (ReactContext) ((TintContextWrapper) context).getBaseContext() : (ReactContext) context;
    }

    @Override
    protected void onSelectionChanged(int selStart, int selEnd) {
        super.onSelectionChanged(selStart, selEnd);

        int min = Math.min(selStart, selEnd);
        int max = Math.max(selStart, selEnd);

        WritableMap selectionRect = Utils.getTextSelectionRect(this, min, max);
        if (selectionRect != null) {
            WritableMap selection = Arguments.createMap();
            selection.putInt("start", min);
            selection.putInt("end", max);
            selection.putMap("rect", selectionRect);

            WritableMap event = Arguments.createMap();
            event.putMap("selection", selection);

            final ReactContext reactContext = getReactContext();
            reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(this.getId(), "topOnTextSelectionChange", event);
        }
    }
}
